# ------------------------------------------------------------------------------
#  Frame .editor.notebook.cyclide
#
#  Methods:
#     - CyclideMenu:Create
#     - CyclideMenu:Init
#     - CyclideMenu:Adjust               = adjust the scale parameters based on value
#
#  Variables:
#     - cyclideGeomFrame
#     - cyclideOptionsFrame
#
#     - majorradius, minorradius, thetamin,thetamax, thetaslices, phimin, phimax, phislices:
#             Value, Min, Max, AbsMin, AbsMax, Step, On, Help
#
#     - cyclideUndoList
#     - cyclideRedoList
#
# ------------------------------------------------------------------------------

proc CyclideMenu:Create { cyclideFrame } {
    global cyclideGeomFrame cyclideOptionsFrame

    #########################################
    # Init
    CyclideMenu:Init

    #########################################
    # Add Widgets to pages of notebook

    set cyclideGeomFrame [Notebook:frame $cyclideFrame Geometry]
    set parameters {majorradius minorradius stringlength thetamin thetamax thetaslices phimin phimax phislices}
    GeomMenu:CreateGeometry cyclide $cyclideGeomFrame $parameters

    set cyclideOptionsFrame [Notebook:frame $cyclideFrame Options]
    set options {surface shading lod solid}
    GeomMenu:CreateOptions $cyclideOptionsFrame $options
}

proc CyclideMenu:Init { } {
    global cyclideUndoList cyclideRedoList

    set cyclideUndoList {}
    set cyclideRedoList {}

    global majorradiusValue majorradiusMin majorradiusMax majorradiusAbsMin \
	    majorradiusAbsMax majorradiusStep majorradiusOn majorradiusHelp \
	    cyclidemajorradiusSliderInit
    global minorradiusValue minorradiusMin minorradiusMax minorradiusAbsMin \
	    minorradiusAbsMax minorradiusStep minorradiusOn minorradiusHelp \
	    cyclideminorradiusSliderInit
    global stringlengthValue stringlengthMin stringlengthMax stringlengthAbsMin \
	    stringlengthAbsMax stringlengthStep stringlengthOn stringlengthHelp \
	    cyclidestringlengthSliderInit
    global thetaminValue thetaminMin thetaminMax thetaminAbsMin thetaminAbsMax \
	    thetaminStep thetaminOn thetaminHelp cyclidethetaminSliderInit
    global thetamaxValue thetamaxMin thetamaxMax thetamaxAbsMin thetamaxAbsMax \
	    thetamaxStep thetamaxOn thetamaxHelp cyclidethetamaxSliderInit
    global thetaslicesValue thetaslicesMin thetaslicesMax thetaslicesHelp\
	    thetaslicesAbsMin thetaslicesAbsMax thetaslicesStep thetaslicesOn  \
	    cyclidethetaslicesSliderInit
    global phiminValue phiminMin phiminMax phiminAbsMin phiminAbsMax \
	    phiminStep phiminOn phiminHelp cyclidephiminSliderInit
    global phimaxValue phimaxMin phimaxMax phimaxAbsMin phimaxAbsMax \
	    phimaxStep phimaxOn phimaxHelp cyclidephimaxSliderInit
    global phislicesValue phislicesMin phislicesMax phislicesHelp\
	    phislicesAbsMin phislicesAbsMax phislicesStep phislicesOn  \
	    cyclidephislicesSliderInit

    global solidValue solidList solidHelp
    global shadingValue shadingList shadingHelp
    global lodValue lodList lodHelp

    global ribbeginValue ribbeginHelp
    global ribendValue ribendHelp
  
    set majorradiusValue   0.5
    set majorradiusMin     0.0
    set majorradiusMax    10.0
    set majorradiusAbsMin  0.0
    set majorradiusAbsMax 1000000.0
    set majorradiusStep    1.0
    set majorradiusOn 0
    set majorradiusHelp "major radius of the cyclide"
    set cyclidemajorradiusSliderInit 0

    set minorradiusValue   0.4
    set minorradiusMin     0.0
    set minorradiusMax    10.0
    set minorradiusAbsMin  0.0
    set minorradiusAbsMax 1000000.0
    set minorradiusStep    1.0
    set minorradiusOn 0
    set minorradiusHelp "minor radius of the cyclide"
    set cyclideminorradiusSliderInit 0

    set stringlengthValue   0.8
    set stringlengthMin     0.0
    set stringlengthMax    10.0
    set stringlengthAbsMin  0.0
    set stringlengthAbsMax 1000000.0
    set stringlengthStep    1.0
    set stringlengthOn 0
    set stringlengthHelp "string length of the cyclide"
    set cyclidestringlengthSliderInit 0
  
    set thetaminValue   0
    set thetaminMin    -360.0
    set thetaminMax     360.0
    set thetaminAbsMin -360.0
    set thetaminAbsMax  360.0
    set thetaminStep 1.0
    set thetaminOn 0
    set thetaminHelp "the string is pulled taught around the cyclide
ellipse between angle thetamin and thetamax from the x-axis."
    set cyclidethetaminSliderInit 0

    set thetamaxValue   360
    set thetamaxMin    -360.0
    set thetamaxMax     360.0
    set thetamaxAbsMin -360.0
    set thetamaxAbsMax  360.0
    set thetamaxStep 1.0
    set thetamaxOn 0
    set thetamaxHelp "the string is pulled taught around the cyclide
ellipse between angle thetamin and thetamax from the x-axis."
    set cyclidethetamaxSliderInit 0

    set thetaslicesValue    8
    set thetaslicesMin      0
    set thetaslicesMax    100
    set thetaslicesAbsMin   0
    set thetaslicesAbsMax 1000000
    set thetaslicesStep 1
    set thetaslicesOn 0
    set thetaslicesHelp "number of triangle strips used to tesselate 
the cyclide in the radial direction"
    set cyclidethetaslicesSliderInit 0

    set phiminValue   0
    set phiminMin    -360.0
    set phiminMax     360.0
    set phiminAbsMin -360.0
    set phiminAbsMax  360.0
    set phiminStep 1.0
    set phiminOn 0
    set phiminHelp "The string is swept around the cyclide ellipse 
between angle phimin to angle phimax."
    set cyclidephiminSliderInit 0

    set phimaxValue   360
    set phimaxMin    -360.0
    set phimaxMax     360.0
    set phimaxAbsMin -360.0
    set phimaxAbsMax  360.0
    set phimaxStep 1.0
    set phimaxOn 0
    set phimaxHelp "The string is swept around the cyclide ellipse 
between angle phimin to angle phimax."
    set cyclidephimaxSliderInit 0

    set phislicesValue    8
    set phislicesMin      0
    set phislicesMax    100
    set phislicesAbsMin   0
    set phislicesAbsMax 1000000
    set phislicesStep 1
    set phislicesOn 0
    set phislicesHelp "number of triangle strips used to tesselate 
the cyclide around the cross section"
    set cyclidephislicesSliderInit 0

    set solidValue solid
    set solidList {solid hollow}
    set solidHelp "This option determines whether backface culling is on or off:
   solid = enable backface culling
   hollow = disable backface culling"

    set shadingValue inherit
    set shadingList {inherit wire flat gouraud phong}
    set shadingHelp "This option defines how the polygons of the object are shaded:
   inherit = render in the same way as my parent
   wire = render a wireframe
   flat = render with flat shading 
   gouraud = render with Gouraud shading
   phong = render with Phong shading"

    set lodValue full
    set lodList {off bound edges full}
    set lodHelp "This option defines the quality of the rendering for this subtree:
   off = do not render
   bound = only render the bounding box
   edges = render in wireframe mode
   full = fully render"

    set ribbeginValue ""
    set ribbeginHelp "Renderman specific ASCII text that is written just 
before the primitive when outputting a RIB file"

    set ribendValue ""
    set ribendHelp "Renderman specific ASCII text that is written just 
after the primitive when outputting a RIB file"
}

