# ------------------------------------------------------------------------------
#  Frame .editor.notebook.cylinder
#
#  Methods:
#     - CylinderMenu:Create
#     - CylinderMenu:Init
#     - CylinderMenu:Adjust               = adjust the scale parameters based on value
#
#  Variables:
#     - cylinderGeomFrame
#     - cylinderOptionsFrame
#
#     - radius, zmin, zmax, zslices, thetamax, thetaslices:
#             Value, Min, Max, AbsMin, AbsMax, Step, On, Help
#
#     - cylinderUndoList
#     - cylinderRedoList
#
# ------------------------------------------------------------------------------

proc CylinderMenu:Create { cylinderFrame } {
    global cylinderGeomFrame cylinderOptionsFrame

    #########################################
    # Init
    CylinderMenu:Init

    #########################################
    # Add Widgets to pages of notebook

    set cylinderGeomFrame [Notebook:frame $cylinderFrame Geometry]
    set parameters {radius zmin zmax zslices thetamax thetaslices}
    GeomMenu:CreateGeometry cylinder $cylinderGeomFrame $parameters

    set cylinderOptionsFrame [Notebook:frame $cylinderFrame Options]
    set options {surface shading lod solid}
    GeomMenu:CreateOptions $cylinderOptionsFrame $options
}

proc CylinderMenu:Init { } {
    global cylinderUndoList cylinderRedoList

    set cylinderUndoList {}
    set cylinderRedoList {}

    global radiusValue radiusMin radiusMax radiusAbsMin radiusAbsMax \
	    radiusStep radiusOn radiusHelp cylinderradiusSliderInit
    global zminValue zminMin zminMax zminAbsMin zminAbsMax \
	    zminStep zminOn zminHelp cylinderzminSliderInit
    global zmaxValue zmaxMin zmaxMax zmaxAbsMin zmaxAbsMax \
	    zmaxStep zmaxOn zmaxHelp cylinderzmaxSliderInit
    global zslicesValue zslicesMin zslicesMax zslicesAbsMin zslicesAbsMax \
	    zslicesStep zslicesOn zslicesHelp cylinderzslicesSliderInit
    global thetamaxValue thetamaxMin thetamaxMax thetamaxAbsMin thetamaxAbsMax \
	    thetamaxStep thetamaxOn thetamaxHelp cylinderthetamaxSliderInit
    global thetaslicesValue thetaslicesMin thetaslicesMax thetaslicesHelp\
	    thetaslicesAbsMin thetaslicesAbsMax thetaslicesStep thetaslicesOn  \
	    cylinderthetaslicesSliderInit

    global solidValue solidList solidHelp
    global shadingValue shadingList shadingHelp
    global lodValue lodList lodHelp

    global ribbeginValue ribbeginHelp
    global ribendValue ribendHelp
  
    set radiusValue   1.0
    set radiusMin     0.0
    set radiusMax    10.0
    set radiusAbsMin  0.0
    set radiusAbsMax 1000000.0
    set radiusStep    1.0
    set radiusOn 0
    set radiusHelp "radius of the cylinder"
    set cylinderradiusSliderInit 0
  
    set zminValue  0.0
    set zminMin    0.0
    set zminMax    10.0
    set zminAbsMin -1000000.0
    set zminAbsMax 1000000.0
    set zminStep   1.0
    set zminOn 0
    set zminHelp "minimum latitude of the cylinder along the z-axis"
    set cylinderzminSliderInit 0    

    set zmaxValue  1.0
    set zmaxMin    0.0
    set zmaxMax    10.0
    set zmaxAbsMin -1000000.0
    set zmaxAbsMax 1000000.0
    set zmaxStep   1.0
    set zmaxOn 0
    set zmaxHelp "maximum latitude of the cylinder along the z-axis"
    set cylinderzmaxSliderInit 0

    set zslicesValue    8
    set zslicesMin      0
    set zslicesMax    100
    set zslicesAbsMin   0
    set zslicesAbsMax 1000000
    set zslicesStep 1
    set zslicesOn 0
    set zslicesHelp "number of triangle strips used to tesselate 
the cylinder in the z-direction"
    set cylinderzslicesSliderInit 0

    set thetamaxValue   360
    set thetamaxMin    -360.0
    set thetamaxMax     360.0
    set thetamaxAbsMin -360.0
    set thetamaxAbsMax  360.0
    set thetamaxStep 1.0
    set thetamaxOn 0
    set thetamaxHelp "the cylinder begins at the x-axis and 
circles the z-axis by thetamax degrees"
    set cylinderthetamaxSliderInit 0

    set thetaslicesValue    8
    set thetaslicesMin      0
    set thetaslicesMax    100
    set thetaslicesAbsMin   0
    set thetaslicesAbsMax 1000000
    set thetaslicesStep 1
    set thetaslicesOn 0
    set thetaslicesHelp "number of triangle strips used to tesselate 
the cylinder in the radial direction"
    set cylinderthetaslicesSliderInit 0

    set solidValue solid
    set solidList {solid hollow}
    set solidHelp "This option determines whether backface culling is on or off:
   solid = enable backface culling
   hollow = disable backface culling"

    set shadingValue inherit
    set shadingList {inherit wire flat gouraud phong}
    set shadingHelp "This option defines how the polygons of the object are shaded:
   inherit = render in the same way as my parent
   wire = render a wireframe
   flat = render with flat shading 
   gouraud = render with Gouraud shading
   phong = render with Phong shading"

    set lodValue full
    set lodList {off bound edges full}
    set lodHelp "This option defines the quality of the rendering for this subtree:
   off = do not render
   bound = only render the bounding box
   edges = render in wireframe mode
   full = fully render"

    set ribbeginValue ""
    set ribbeginHelp "Renderman specific ASCII text that is written just 
before the primitive when outputting a RIB file"

    set ribendValue ""
    set ribendHelp "Renderman specific ASCII text that is written just 
after the primitive when outputting a RIB file"
}

