# ------------------------------------------------------------------------------
#  Frame: .editor.notebook.group
#
#  Methods:
#     - GroupMenu:Create
#     - GroupMenu:Init
#     - GroupMenu:CreateOptions
#     - GroupMenu:CreateInsts
#     - GroupMenu:SetParameterValues
#
#  Variables:
#     - groupInstsFrame
#     - groupOptionsFrame
#
# ------------------------------------------------------------------------------

proc GroupMenu:Create { groupFrame } {
    global groupInstsFrame groupOptionsFrame

    #########################################
    # Init
    GroupMenu:Init

    #########################################
    # Add Widgets to pages of notebook

    set groupInstsFrame [Notebook:frame $groupFrame Instances]
    GroupMenu:CreateInsts $groupInstsFrame

    set groupOptionsFrame [Notebook:frame $groupFrame Options]
    GroupMenu:CreateOptions $groupOptionsFrame
}

proc GroupMenu:Init { } {
    global shadingValue shadingList shadingHelp
    global lodValue lodList lodHelp
    global ribbeginValue ribbeginHelp
    global ribendValue ribendHelp
    
    set shadingValue inherit
    set shadingList {inherit wire flat gouraud phong}
    set shadingHelp "This option defines how the polygons of the object are shaded:
   inherit = render in the same way as my parent
   wire = render a wireframe
   flat = render with flat shading 
   gouraud = render with Gouraud shading
   phong = render with Phong shading"

    set lodValue full
    set lodList {off bound edges full}
    set lodHelp "This option defines the quality of the rendering for this subtree:
   off = do not render
   bound = only render the bounding box
   edges = render in wireframe mode
   full = fully render"

    set ribbeginValue ""
    set ribbeginHelp "Renderman specific ASCII text that is written just 
before the primitive when outputting a RIB file"

    set ribendValue ""
    set ribendHelp "Renderman specific ASCII text that is written just 
after the primitive when outputting a RIB file"
}


proc GroupMenu:CreateOptions { base } {

    set options {surface shading lod}

    set y 20
    foreach option $options {

	CreateOptionParameterFrame $base $option $y
	incr y 60
    }
    incr y 60
    CreateRibEntryFrame $base $y
}

proc GroupMenu:CreateInsts { base } {
    global labelFont

    set y 20
    set x 50
      
    Label $base.label -borderwidth 1 -font $labelFont -text "instances:" \
	    -helptext "list of nodes that have been instantiated by the group"
    Label $base.instances -borderwidth 1 -font $labelFont -text "" -justify left
  
    place $base.label -x 10 -y $y -anchor nw -bordermode ignore 
    place $base.instances -x 100 -y $y -anchor nw -bordermode ignore 
}

proc GroupMenu:SetParameterValues { paramValues } {
    global surfaceValue shadingValue lodValue ribbeginValue ribendValue
    global groupInstsFrame

    foreach pair $paramValues {

	set param [lindex $pair 0]
	set value [lindex $pair 1]

	if { $value == "(null)" } {
	    set value ""
	}

	if { $param == "instances" } {

	    set insts ""
	    foreach inst $value {
		set insts "$insts $inst\n"
	    }
	    
	    $groupInstsFrame.instances configure -text $insts

	} else {
	    #puts "$param = $value"	
	    set [subst $param]Value $value
	}
    }
}

