# ------------------------------------------------------------------------------
#  Frame .editor.notebook.sphere
#
#  Methods:
#     - SphereMenu:Create
#     - SphereMenu:Init
#     - SphereMenu:CreateGeometry
#     - SphereMenu:CreateOptions
#
#  Variables:
#     - sphereGeomFrame
#     - sphereOptionsFrame
#
#     - radius, zmin, zmax, zslices, thetamax, thetaslices:
#             Value, Min, Max, AbsMin, AbsMax, Step, On, Help
#
#     - sphereUndoList
#     - sphereRedoList
#
# ------------------------------------------------------------------------------

proc SphereMenu:Create { sphereFrame } {
    global sphereGeomFrame sphereOptionsFrame

    #########################################
    # Init
    SphereMenu:Init

    #########################################
    # Add Widgets to pages of notebook

    set sphereGeomFrame [Notebook:frame $sphereFrame Geometry]
    set parameters {radius zmin zmax zslices thetamax thetaslices}
    GeomMenu:CreateGeometry sphere $sphereGeomFrame $parameters

    set sphereOptionsFrame [Notebook:frame $sphereFrame Options]
    set options {surface shading lod solid}
    GeomMenu:CreateOptions $sphereOptionsFrame $options
}

proc SphereMenu:Init { } {
    global sphereUndoList sphereRedoList

    set sphereUndoList {}
    set sphereRedoList {}

    global radiusValue radiusMin radiusMax radiusAbsMin radiusAbsMax \
	    radiusStep radiusOn radiusHelp sphereradiusSliderInit
    global zminValue zminMin zminMax zminAbsMin zminAbsMax \
	    zminStep zminOn zminHelp spherezminSliderInit
    global zmaxValue zmaxMin zmaxMax zmaxAbsMin zmaxAbsMax \
	    zmaxStep zmaxOn zmaxHelp spherezmaxSliderInit
    global zslicesValue zslicesMin zslicesMax zslicesAbsMin zslicesAbsMax \
	    zslicesStep zslicesOn zslicesHelp spherezslicesSliderInit
    global thetamaxValue thetamaxMin thetamaxMax thetamaxAbsMin thetamaxAbsMax \
	    thetamaxStep thetamaxOn thetamaxHelp spherethetamaxSliderInit
    global thetaslicesValue thetaslicesMin thetaslicesMax thetaslicesHelp\
	    thetaslicesAbsMin thetaslicesAbsMax thetaslicesStep thetaslicesOn  \
	    spherethetaslicesSliderInit

    global solidValue solidList solidHelp
    global shadingValue shadingList shadingHelp
    global lodValue lodList lodHelp

    global ribbeginValue ribbeginHelp
    global ribendValue ribendHelp
  
    set radiusValue   1.0
    set radiusMin     0.0
    set radiusMax    10.0
    set radiusAbsMin  0.0
    set radiusAbsMax 1000000.0
    set radiusStep    1.0
    set radiusOn 0
    set radiusHelp "radius of the sphere"
    set sphereradiusSliderInit 0

    set zminValue  0.0
    set zminMin    0.0
    set zminMax    1.0
    set zminAbsMin 0.0
    set zminAbsMax 1.0
    set zminStep   0.01
    set zminOn 0
    set zminHelp "minimum latitude of the sphere along the z-axis
as a percentage of the diameter"
    set spherezminSliderInit 0    

    set zmaxValue  1.0
    set zmaxMin    0.0
    set zmaxMax    1.0
    set zmaxAbsMin 0.0
    set zmaxAbsMax 1.0
    set zmaxStep   0.01
    set zmaxOn 0
    set zmaxHelp "maximum latitude of the sphere along the z-axis
as a percentage of the diameter"
    set spherezmaxSliderInit 0

    set zslicesValue    8
    set zslicesMin      0
    set zslicesMax    100
    set zslicesAbsMin   0
    set zslicesAbsMax 1000000
    set zslicesStep 1
    set zslicesOn 0
    set zslicesHelp "number of longitudinal triangle strips 
used to tesselate the sphere"
    set spherezslicesSliderInit 0

    set thetamaxValue   360
    set thetamaxMin    -360.0
    set thetamaxMax     360.0
    set thetamaxAbsMin -360.0
    set thetamaxAbsMax  360.0
    set thetamaxStep 1.0
    set thetamaxOn 0
    set thetamaxHelp "the sphere begins at the x-axis and 
circles the z-axis by thetamax degrees"
    set spherethetamaxSliderInit 0

    set thetaslicesValue    8
    set thetaslicesMin      0
    set thetaslicesMax    100
    set thetaslicesAbsMin   0
    set thetaslicesAbsMax 1000000
    set thetaslicesStep 1
    set thetaslicesOn 0
    set thetaslicesHelp "number of latitudinal triangle strips 
used to tesselate the sphere"
    set spherethetaslicesSliderInit 0

    set solidValue solid
    set solidList {solid hollow}
    set solidHelp "This option determines whether backface culling is on or off:
   solid = enable backface culling
   hollow = disable backface culling"

    set shadingValue inherit
    set shadingList {inherit wire flat gouraud phong}
    set shadingHelp "This option defines how the polygons of the object are shaded:
   inherit = render in the same way as my parent
   wire = render a wireframe
   flat = render with flat shading 
   gouraud = render with Gouraud shading
   phong = render with Phong shading"

    set lodValue full
    set lodList {off bound edges full}
    set lodHelp "This option defines the quality of the rendering for this subtree:
   off = do not render
   bound = only render the bounding box
   edges = render in wireframe mode
   full = fully render"

    set ribbeginValue ""
    set ribbeginHelp "Renderman specific ASCII text that is written just 
before the primitive when outputting a RIB file"

    set ribendValue ""
    set ribendHelp "Renderman specific ASCII text that is written just 
after the primitive when outputting a RIB file"
}

