######################################################################
# fractals.tcl - Fractal contours

#source "matrix.tcl"

proc KochSnowflake { uiGeneration } {

    global SLF_PI
    set fThetaPercent 0.5

    set pppfPath { { { 0.0 0.0 0.0 1.0 } { -1.0 0.0 0.0 1.0 } } }

    set pmPath {}

    set mRotate [MatrixRotateZ [expr 0.0]]
    set mTranslate [MatrixTranslate [list 0.0 0.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/3.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mRotate [MatrixRotateZ [expr -$SLF_PI/3.0]]
    set mTranslate [MatrixTranslate [list -1.0 0.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/3.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mScale [MatrixScaleAll [expr 1.0/3.0]]
    set mRotate [MatrixRotateZ [expr $SLF_PI/3.0]]
    set mTranslate [MatrixTranslate [list -0.5 [expr sqrt(3.0)/6.0] 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mRotate $mScale]]

    set mScale [MatrixScaleAll [expr 1.0/3.0]]
    set mRotate [MatrixRotateZ 0.0]
    set mTranslate [MatrixTranslate [list [expr -2.0/3.0] 0.0 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mRotate $mScale]]

    for { set uiGen 1 } { $uiGen <= $uiGeneration } { incr uiGen } {
	set uiGenPrev [expr $uiGen - 1]

	set pathPrev [lindex $pppfPath $uiGenPrev]
	set uiVertices [llength $pathPrev]
	set pathCurr { { 0.0 0.0 0.0 1.0 } }
	for { set uiSegment 0 } { $uiSegment < 4 } { incr uiSegment } {
	    for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
		lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	    }
	}
	lappend pppfPath $pathCurr
    }

    ##################################################
    # Instance the edge three times to form a contour

    set ppfContour {}
    set ppfPath [lindex $pppfPath $uiGeneration]
    set uiVertices [llength $ppfPath]

    set fTheta [expr $SLF_PI/6.0]
    set fThetaDelta [expr 2.0*$SLF_PI/3.0]
    set fPhi [expr -$SLF_PI/6.0]

    for { set uiSegment 0 } { $uiSegment < 3 } { incr uiSegment } {
	set mRotateA [MatrixRotateZ $fPhi]
	set mTranslate [MatrixTranslate [list [expr 1.0/sqrt(3.0)] 0.0 0.0 0.0]]
	set mRotateB [MatrixRotateZ $fTheta]

	set mContour [MatrixMultMatrix $mRotateB [MatrixMultMatrix $mTranslate $mRotateA]]

	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    #puts "Vertex $uiVertex: [lindex $ppfPath $uiVertex]]"
	    lappend ppfContour [MatrixMultMatrix $mContour [lindex $ppfPath $uiVertex]]
	}

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return $ppfContour
}

proc KochSnowflakeExterior { uiGeneration } {

    global SLF_PI
    set fThetaPercent 0.5

    set pppfPath { { { 0.0 0.0 0.0 1.0 } { 1.0 0.0 0.0 1.0 } } }

    set pmPath {}

    set mRotate [MatrixRotateZ [expr 0.0]]
    set mTranslate [MatrixTranslate [list 0.0 0.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/3.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mRotate [MatrixRotateZ [expr $SLF_PI/3.0]]
    set mTranslate [MatrixTranslate [list 1.0 0.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/3.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mScale [MatrixScaleAll [expr 1.0/3.0]]
    set mRotate [MatrixRotateZ [expr -$SLF_PI/3.0]]
    set mTranslate [MatrixTranslate [list 0.5 [expr sqrt(3.0)/6.0] 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mRotate $mScale]]

    set mScale [MatrixScaleAll [expr 1.0/3.0]]
    set mRotate [MatrixRotateZ 0.0]
    set mTranslate [MatrixTranslate [list [expr 2.0/3.0] 0.0 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mRotate $mScale]]

    for { set uiGen 1 } { $uiGen <= $uiGeneration } { incr uiGen } {
	set uiGenPrev [expr $uiGen - 1]

	set pathPrev [lindex $pppfPath $uiGenPrev]
	set uiVertices [llength $pathPrev]
	set pathCurr { { 0.0 0.0 0.0 1.0 } }
	for { set uiSegment 0 } { $uiSegment < 4 } { incr uiSegment } {
	    for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
		lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	    }
	}
	lappend pppfPath $pathCurr
    }

    ##################################################
    # Instance the edge three times to form a contour

    set ppfContour {}
    set ppfPath [lindex $pppfPath $uiGeneration]
    set uiVertices [llength $ppfPath]

    set fTheta [expr 0.0]
    set fThetaDelta [expr $SLF_PI/3.0]
    set fPhi [expr 2.0*$SLF_PI/3.0]

    for { set uiSegment 0 } { $uiSegment < 6 } { incr uiSegment } {
	set mRotateA [MatrixRotateZ $fPhi]
	set mTranslate [MatrixTranslate [list [expr 1.0] 0.0 0.0 0.0]]
	set mRotateB [MatrixRotateZ $fTheta]

	set mContour [MatrixMultMatrix $mRotateB [MatrixMultMatrix $mTranslate $mRotateA]]

	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    #puts "Vertex $uiVertex: [lindex $ppfPath $uiVertex]]"
	    lappend ppfContour [MatrixMultMatrix $mContour [lindex $ppfPath $uiVertex]]
	}

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return $ppfContour
}

proc PeanoGosper { uiGeneration } {

    global SLF_PI
    set fThetaPercent 0.5

    set pppfPath { { { 0.0 0.0 0.0 1.0 } { -1.0 0.0 0.0 1.0 } } }

    set pmPath {}

    set mTranslate [MatrixTranslate [list 1.0 0.0 0.0 0.0]]
    set mRotateA [MatrixRotateZ [expr 2.0*$SLF_PI/3.0]]
    set mRotate [MatrixRotateZ [expr -atan2(sqrt(3.0), 5.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(7.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate [MatrixMultMatrix $mRotateA $mTranslate]]]

    set mTranslate [MatrixTranslate [list [expr -1.0/2.0] [expr sqrt(3.0)/2.0] 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(sqrt(3.0), 5.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(7.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate $mTranslate]]

    set mTranslate [MatrixTranslate [list [expr -3.0/2.0] [expr sqrt(3.0)/2.0] 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(sqrt(3.0), 5.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(7.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate $mTranslate]]

    set mTranslateA [MatrixTranslate [list 1.0 0.0 0.0 0.0]]
    set mRotateA [MatrixRotateZ [expr 2.0*$SLF_PI/3.0]]
    set mTranslateB [MatrixTranslate [list -2.0 0.0 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(sqrt(3.0), 5.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(7.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate [MatrixMultMatrix $mTranslateB [MatrixMultMatrix $mRotateA $mTranslateA]]]]

    set mTranslate [MatrixTranslate [list -1.0 0.0 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(sqrt(3.0), 5.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(7.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate $mTranslate]]

    set mTranslateA [MatrixTranslate [list 1.0 0.0 0.0 0.0]]
    set mRotateA [MatrixRotateZ [expr -2.0*$SLF_PI/3.0]]
    set mTranslateB [MatrixTranslate [list -1.0 0.0 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(sqrt(3.0), 5.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(7.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate [MatrixMultMatrix $mTranslateB [MatrixMultMatrix $mRotateA $mTranslateA]]]]

    set mTranslate [MatrixTranslate [list [expr -3.0/2.0] [expr -sqrt(3.0)/2.0] 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(sqrt(3.0), 5.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(7.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate $mTranslate]]

    for { set uiGen 1 } { $uiGen <= $uiGeneration } { incr uiGen } {
	set uiGenPrev [expr $uiGen - 1]

	set pathPrev [lindex $pppfPath $uiGenPrev]
	set uiVertices [llength $pathPrev]
	set pathCurr { { 0.0 0.0 0.0 1.0 } }

	set uiSegment 0
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev [expr $uiVertices - 1 - $uiVertex]]]
	}

	set uiEnd $uiVertices
	if { $uiGen == 1 } {
	    incr uiEnd -1
	}
	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiEnd } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev [expr $uiVertices - 1 - $uiVertex]]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev [expr $uiVertices - 1 - $uiVertex]]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	lappend pppfPath $pathCurr
    }

    ##################################################
    # Instance the edge six times to form a contour

    set ppfContour {}
    set ppfPath [lindex $pppfPath $uiGeneration]
    set uiVertices [llength $ppfPath]

    set fTheta [expr 0.0]
    set fThetaDelta [expr $SLF_PI/3.0]
    set fPhi [expr -$SLF_PI/3.0]

    for { set uiSegment 0 } { $uiSegment < 6 } { incr uiSegment } {
	set mRotateA [MatrixRotateZ $fPhi]
	set mTranslate [MatrixTranslate [list 1.0 0.0 0.0 0.0]]
	set mRotateB [MatrixRotateZ $fTheta]

	set mContour [MatrixMultMatrix $mRotateB [MatrixMultMatrix $mTranslate $mRotateA]]

	for { set uiVertex 1 } { $uiVertex < ($uiVertices - 1) } { incr uiVertex } {
	    #puts "Vertex $uiVertex: [lindex $ppfPath $uiVertex]]"
	    lappend ppfContour [MatrixMultMatrix $mContour [lindex $ppfPath $uiVertex]]
	}

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return $ppfContour
}

proc KochIslandQuadratic { uiGeneration } {

    global SLF_PI

    set pppfPath { { { 0.0 0.0 0.0 1.0 } { 1.0 0.0 0.0 1.0 } } }

    set pmPath {}

    set mRotate [MatrixRotateZ [expr -atan2(1.0, 4.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(17.0)]]

    lappend pmPath [MatrixMultMatrix $mScale $mRotate]

    set mRotateA [MatrixRotateZ [expr -$SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 1.0 0.0 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(1.0, 4.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(17.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate [MatrixMultMatrix $mTranslate $mRotateA]]]

    set mTranslate [MatrixTranslate [list 1.0 -1.0 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(1.0, 4.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(17.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate $mTranslate]]

    set mRotateA [MatrixRotateZ [expr $SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 2.0 -1.0 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(1.0, 4.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(17.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate [MatrixMultMatrix $mTranslate $mRotateA]]]

    set mRotateA [MatrixRotateZ [expr $SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 2.0 0.0 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(1.0, 4.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(17.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate [MatrixMultMatrix $mTranslate $mRotateA]]]

    set mRotateA [MatrixRotateZ [expr $SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 2.0 1.0 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(1.0, 4.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(17.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate [MatrixMultMatrix $mTranslate $mRotateA]]]

    set mTranslate [MatrixTranslate [list 2.0 2.0 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(1.0, 4.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(17.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate $mTranslate]]

    set mRotateA [MatrixRotateZ [expr -$SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 3.0 2.0 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(1.0, 4.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(17.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate [MatrixMultMatrix $mTranslate $mRotateA]]]

    set mTranslate [MatrixTranslate [list 3.0 1.0 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr -atan2(1.0, 4.0)]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(17.0)]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate $mTranslate]]

    for { set uiGen 1 } { $uiGen <= $uiGeneration } { incr uiGen } {
	set uiGenPrev [expr $uiGen - 1]

	set pathPrev [lindex $pppfPath $uiGenPrev]
	set uiVertices [llength $pathPrev]
	set pathCurr { { 0.0 0.0 0.0 1.0 } }

	set uiSegment 0
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < ($uiVertices - 1) } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < ($uiVertices -1) } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	lappend pppfPath $pathCurr
    }

    ##################################################
    # Instance the edge four times to form a contour

    set ppfContour {}
    set ppfPath [lindex $pppfPath $uiGeneration]
    set uiVertices [llength $ppfPath]

    set fTheta [expr $SLF_PI/4.0]
    set fThetaDelta [expr $SLF_PI/2.0]
    set fPhi [expr 3.0*$SLF_PI/4.0]

    for { set uiSegment 0 } { $uiSegment < 4 } { incr uiSegment } {
	set mRotateA [MatrixRotateZ $fPhi]
	set mTranslate [MatrixTranslate [list [expr 1.0/sqrt(2.0)] 0.0 0.0 0.0]]
	set mRotateB [MatrixRotateZ $fTheta]

	set mContour [MatrixMultMatrix $mRotateB [MatrixMultMatrix $mTranslate $mRotateA]]

	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    #puts "Vertex $uiVertex: [lindex $ppfPath $uiVertex]]"
	    lappend ppfContour [MatrixMultMatrix $mContour [lindex $ppfPath $uiVertex]]
	}

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return $ppfContour
}

proc FractalContour32 { uiGeneration } {

    global SLF_PI

    set pppfPath { { { 0.0 0.0 0.0 1.0 } { 1.0 0.0 0.0 1.0 } } }

    set pmPath {}

    set mRotate [MatrixRotateZ [expr -$SLF_PI/2.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale $mRotate]

    set mTranslate [MatrixTranslate [list 0.0 -1.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale $mTranslate]

    set mRotate [MatrixRotateZ [expr -$SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 1.0 -1.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mRotate [MatrixRotateZ [expr $SLF_PI]]
    set mTranslate [MatrixTranslate [list 1.0 -2.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mRotate [MatrixRotateZ [expr -$SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 0.0 -2.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mTranslate [MatrixTranslate [list 0.0 -3.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale $mTranslate]

    set mTranslate [MatrixTranslate [list 1.0 -3.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale $mTranslate]

    set mRotate [MatrixRotateZ [expr -$SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 2.0 -3.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mTranslate [MatrixTranslate [list 2.0 -4.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale $mTranslate]

    set mRotate [MatrixRotateZ [expr $SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 3.0 -4.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mRotate [MatrixRotateZ [expr $SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 3.0 -3.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mRotate [MatrixRotateZ [expr $SLF_PI]]
    set mTranslate [MatrixTranslate [list 3.0 -2.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mRotate [MatrixRotateZ [expr $SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 2.0 -2.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mTranslate [MatrixTranslate [list 2.0 -1.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale $mTranslate]

    set mTranslate [MatrixTranslate [list 3.0 -1.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale $mTranslate]

    set mRotate [MatrixRotateZ [expr $SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 4.0 -1.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mRotate [MatrixRotateZ [expr $SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 4.0 0.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mTranslate [MatrixTranslate [list 4.0 1.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale $mTranslate]

    set mTranslate [MatrixTranslate [list 5.0 1.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale $mTranslate]

    set mRotate [MatrixRotateZ [expr $SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 6.0 1.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mRotate [MatrixRotateZ [expr $SLF_PI]]
    set mTranslate [MatrixTranslate [list 6.0 2.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mRotate [MatrixRotateZ [expr $SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 5.0 2.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mRotate [MatrixRotateZ [expr $SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 5.0 3.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mTranslate [MatrixTranslate [list 5.0 4.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale $mTranslate]

    set mRotate [MatrixRotateZ [expr -$SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 6.0 4.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mTranslate [MatrixTranslate [list 6.0 3.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale $mTranslate]

    set mTranslate [MatrixTranslate [list 7.0 3.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale $mTranslate]

    set mRotate [MatrixRotateZ [expr -$SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 8.0 3.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mRotate [MatrixRotateZ [expr $SLF_PI]]
    set mTranslate [MatrixTranslate [list 8.0 2.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mRotate [MatrixRotateZ [expr -$SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 7.0 2.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set mTranslate [MatrixTranslate [list 7.0 1.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale $mTranslate]

    set mRotate [MatrixRotateZ [expr -$SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 8.0 1.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/8.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

    set pbDropLastVertex { 0 0 0 0 0 1 0 0 0 1 0 0 0 1 0 1 0 1 0 0 0 1 0 0 0 1 0 0 0 0 0 0 }

    for { set uiGen 1 } { $uiGen <= $uiGeneration } { incr uiGen } {
	set uiGenPrev [expr $uiGen - 1]

	set pathPrev [lindex $pppfPath $uiGenPrev]
	set uiVertices [llength $pathPrev]
	set pathCurr { { 0.0 0.0 0.0 1.0 } }

	set uiSegment 0
	for { set uiSegment 0 } { $uiSegment < 32 } { incr uiSegment } {
	    set uiEnd $uiVertices
	    if { [lindex $pbDropLastVertex $uiSegment] } {
		incr uiEnd -1
	    }
	    for { set uiVertex 1 } { $uiVertex < $uiEnd } { incr uiVertex } {
		lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	    }
	}

	lappend pppfPath $pathCurr
    }

    ##################################################
    # Instance the edge four times to form a contour

    set ppfContour {}
    set ppfPath [lindex $pppfPath $uiGeneration]
    set uiVertices [llength $ppfPath]

    set fTheta [expr $SLF_PI/4.0]
    set fThetaDelta [expr $SLF_PI/2.0]
    set fPhi [expr 3.0*$SLF_PI/4.0]

    for { set uiSegment 0 } { $uiSegment < 4 } { incr uiSegment } {
	set mRotateA [MatrixRotateZ $fPhi]
	set mTranslate [MatrixTranslate [list [expr 1.0/sqrt(2.0)] 0.0 0.0 0.0]]
	set mRotateB [MatrixRotateZ $fTheta]

	set mContour [MatrixMultMatrix $mRotateB [MatrixMultMatrix $mTranslate $mRotateA]]

	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    #puts "Vertex $uiVertex: [lindex $ppfPath $uiVertex]]"
	    lappend ppfContour [MatrixMultMatrix $mContour [lindex $ppfPath $uiVertex]]
	}

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return $ppfContour
}

proc KochIsland { uiGeneration } {

    global SLF_PI
    set fThetaPercent 0.5

    set pppfPath { { { 0.0 0.0 0.0 1.0 } { 1.0 0.0 0.0 1.0 } } }

    set pmPath {}

    set mScale [MatrixScaleAll 0.25]

    lappend pmPath $mScale

    set mScale [MatrixScaleAll 0.25]
    set mRotate [MatrixRotateZ [expr -$SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 0.25 0.0 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mRotate $mScale]]

    set mScale [MatrixScaleAll 0.25]
    set mTranslate [MatrixTranslate [list 0.25 -0.25 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTranslate $mScale]

    set mScale [MatrixScaleAll 0.25]
    set mRotate [MatrixRotateZ [expr $SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 0.5 -0.25 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mRotate $mScale]]

    set mScale [MatrixScaleAll 0.25]
    set mRotate [MatrixRotateZ [expr $SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 0.5 0.0 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mRotate $mScale]]

    set mScale [MatrixScaleAll 0.25]
    set mTranslate [MatrixTranslate [list 0.5 0.25 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTranslate $mScale]

    set mScale [MatrixScaleAll 0.25]
    set mRotate [MatrixRotateZ [expr -$SLF_PI/2.0]]
    set mTranslate [MatrixTranslate [list 0.75 0.25 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mRotate $mScale]]

    set mScale [MatrixScaleAll 0.25]
    set mTranslate [MatrixTranslate [list 0.75 0.0 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTranslate $mScale]

    for { set uiGen 1 } { $uiGen <= $uiGeneration } { incr uiGen } {
	set uiGenPrev [expr $uiGen - 1]

	set pathPrev [lindex $pppfPath $uiGenPrev]
	set uiVertices [llength $pathPrev]
	set pathCurr { { 0.0 0.0 0.0 1.0 } }
	for { set uiSegment 0 } { $uiSegment < 8 } { incr uiSegment } {
	    set uiEnd $uiVertices
	    if { $uiSegment == 3 } {
		incr uiEnd -1
	    }
	    for { set uiVertex 1 } { $uiVertex < $uiEnd } { incr uiVertex } {
		lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	    }
	}
	lappend pppfPath $pathCurr
    }

    ##################################################
    # Instance the edge four times to form a contour

    set ppfContour {}
    set ppfPath [lindex $pppfPath $uiGeneration]
    set uiVertices [llength $ppfPath]

    set fTheta [expr $SLF_PI/4.0]
    set fThetaDelta [expr $SLF_PI/2.0]
    set fPhi [expr 3.0*$SLF_PI/4.0]

    for { set uiSegment 0 } { $uiSegment < 4 } { incr uiSegment } {
	set mRotateA [MatrixRotateZ $fPhi]
	set mTranslate [MatrixTranslate [list [expr 1.0/sqrt(2.0)] 0.0 0.0 0.0]]
	set mRotateB [MatrixRotateZ $fTheta]

	set mContour [MatrixMultMatrix $mRotateB [MatrixMultMatrix $mTranslate $mRotateA]]

	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    #puts "Vertex $uiVertex: [lindex $ppfPath $uiVertex]]"
	    lappend ppfContour [MatrixMultMatrix $mContour [lindex $ppfPath $uiVertex]]
	}

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return $ppfContour
}

proc SierpinskiArrowhead { uiGeneration } {

    global SLF_PI

    set pppfPath { { { 0.0 0.0 0.0 1.0 } { -1.0 0.0 0.0 1.0 } } }

    set pmPath {}

    set mTranslate [MatrixTranslate [list 1.0 0.0 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr 2.0*$SLF_PI/3.0]]
    set mScale [MatrixScaleAll [expr 0.5]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate $mTranslate]]

    set mTranslate [MatrixTranslate [list [expr -0.5] [expr sqrt(3.0)/2.0] 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 0.5]]

    lappend pmPath [MatrixMultMatrix $mScale $mTranslate]

    set mRotate [MatrixRotateZ [expr -2.0*$SLF_PI/3.0]]
    set mScale [MatrixScaleAll [expr 0.5]]
    set mTranslate [MatrixTranslate [list -1.0 0.0 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mScale $mRotate]]

    for { set uiGen 1 } { $uiGen <= $uiGeneration } { incr uiGen } {
	set uiGenPrev [expr $uiGen - 1]

	set pathPrev [lindex $pppfPath $uiGenPrev]
	set uiVertices [llength $pathPrev]
	set pathCurr { { 0.0 0.0 0.0 1.0 } }

	set uiSegment 0
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev [expr $uiVertices - 1 - $uiVertex]]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev [expr $uiVertices - 1 - $uiVertex]]]
	}

	lappend pppfPath $pathCurr
    }

    ##################################################
    # Instance the edge three times to form a contour

    set ppfContour {}
    set ppfPath [lindex $pppfPath $uiGeneration]
    set uiVertices [llength $ppfPath]

    set fTheta [expr $SLF_PI/6.0]
    set fThetaDelta [expr 2.0*$SLF_PI/3.0]
    set fPhi [expr -$SLF_PI/6.0]

    for { set uiSegment 0 } { $uiSegment < 3 } { incr uiSegment } {
	set mRotateA [MatrixRotateZ $fPhi]
	set mTranslate [MatrixTranslate [list [expr 1.0/sqrt(3.0)] 0.0 0.0 0.0]]
	set mRotateB [MatrixRotateZ $fTheta]

	set mContour [MatrixMultMatrix $mRotateB [MatrixMultMatrix $mTranslate $mRotateA]]

	set uiStart 1
	set uiEnd $uiVertices
	if { ( $uiGeneration & 1 ) } {
	    set uiEnd [expr $uiEnd - 1]
	}

	for { set uiVertex $uiStart } { $uiVertex < $uiEnd } { incr uiVertex } {
	    #puts "Vertex $uiVertex: [lindex $ppfPath $uiVertex]]"
	    lappend ppfContour [MatrixMultMatrix $mContour [lindex $ppfPath $uiVertex]]
	}

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return $ppfContour
}

proc DragonOctagon { uiGeneration } {

    global SLF_PI

    ##################################################
    # The path piece

#    set pppfPath [list \
#                   [list \
#                     [list -0.25 -0.25 0.0 1.0 ] \
#		     [list [expr -tan($SLF_PI/8.0)*sqrt(2.0)/4.0] [expr -sqrt(2.0)/4.0] 0.0 1.0 ] \
#		     [list [expr tan($SLF_PI/8.0)*sqrt(2.0)/4.0] [expr -sqrt(2.0)/4.0] 0.0 1.0 ] \
#		     [list 0.25 -0.25 0.0 1.0 ] \
#                   ] \
#                 ]
    
    set pppfPath [list \
                   [list \
		     [list [expr -tan($SLF_PI/8.0)*sqrt(2.0)/4.0] [expr -sqrt(2.0)/4.0] 0.0 1.0 ] \
		     [list [expr tan($SLF_PI/8.0)*sqrt(2.0)/4.0] [expr -sqrt(2.0)/4.0] 0.0 1.0 ] \
                   ] \
                 ]
    
    set pbPathOrient { 0 0 1 }
    set pmPath {}
	     
    set mTrans0 [MatrixTranslate [list 0.5 0.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(2.0)]]
    set mRotate [MatrixRotateZ [expr -$SLF_PI/4.0]]
    set mTrans1 [MatrixTranslate [list -0.5 0.0 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTrans1 [MatrixMultMatrix $mRotate [MatrixMultMatrix $mScale $mTrans0]]]

    lappend pmPath [MatrixTranslate [list 0.0 -0.5 0.0 0.0]]

    set mTrans0 [MatrixTranslate [list 0.5 0.0 0.0 0.0]]
    set mScale [MatrixScaleAll [expr 1.0/sqrt(2.0)]]
    set mRotate [MatrixRotateZ [expr -3.0*$SLF_PI/4.0]]
    set mTrans1 [MatrixTranslate [list 0.5 0.0 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTrans1 [MatrixMultMatrix $mRotate [MatrixMultMatrix $mScale $mTrans0]]]

    ##################################################
    # The corner piece

    set mTrans [MatrixTranslate [list 0.0 0.5 0.0 0.0]]
    set mRotate [MatrixRotateZ [expr $SLF_PI/2.0]]

    set mCorner [MatrixMultMatrix $mRotate $mTrans]

    set path [lindex $pppfPath 0]
    set uiVertices [llength $path]

    set corner {}
    for { set uiVertex 0 } { $uiVertex < $uiVertices } { incr uiVertex } {
	lappend corner [MatrixMultMatrix $mCorner [lindex $path $uiVertex]]
    }

    set pppfCorner [list $corner]

    set mScale [MatrixScaleAll [expr 1.0/sqrt(2.0)]]
    set mRotate [MatrixRotateZ [expr -$SLF_PI/4.0]]

    set mCorner [MatrixMultMatrix $mRotate $mScale]

    ##################################################
    # The fractal recursion

    for { set uiGen 1 } { $uiGen <= $uiGeneration } { incr uiGen } {
	set uiGenPrev [expr $uiGen - 1]
	
	set ppfPathPrev [lindex $pppfPath $uiGenPrev]
	set uiPathVertices [llength $ppfPathPrev]
	set ppfCornerPrev [lindex $pppfCorner $uiGenPrev]
	set uiCornerVertices [llength $ppfCornerPrev]

	##############################
	# Update the Corner

	set ppfCornerCurr {}
	for { set uiVertex 0 } { $uiVertex < $uiCornerVertices } { incr uiVertex } {
	    lappend ppfCornerCurr [MatrixMultMatrix $mCorner [lindex $ppfCornerPrev $uiVertex]]
	}
	lappend pppfCorner $ppfCornerCurr

	##############################
	# Update the Path

	set uiSegment 0
	set ppfPathCurr {}
	for { set uiVertex 0 } { $uiVertex < $uiPathVertices } { incr uiVertex } {
	    lappend ppfPathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $ppfPathPrev $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 0 } { $uiVertex < $uiCornerVertices } { incr uiVertex } {
	    lappend ppfPathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $ppfCornerCurr $uiVertex]]
	}

	incr uiSegment
	for { set uiVertex 0 } { $uiVertex < $uiPathVertices } { incr uiVertex } {
	    lappend ppfPathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $ppfPathPrev [expr $uiPathVertices - 1 - $uiVertex]]]
	}

	lappend pppfPath $ppfPathCurr
    }

#    puts "Paths:"
#    for { set uiGen 0 } { $uiGen <= $uiGeneration } { incr uiGen } {
#	puts "  Gen $uiGen: [lindex $pppfPath $uiGen]"
#    }

#    puts ""
#    puts "Corners:"
#    for { set uiGen 0 } { $uiGen <= $uiGeneration } { incr uiGen } {
#	puts "  Gen $uiGen: [lindex $pppfCorner $uiGen]"
#    }

    ##############################
    # Create the Half Contour

    set mRotate [MatrixRotateZ [expr $SLF_PI]]
    set mTrans [MatrixTranslate [list -0.5 0.0 0.0 0.0]]
    set mHalf [MatrixMultMatrix $mTrans $mRotate]
    
    set ppfPathPrev [lindex $pppfPath $uiGeneration]
    set uiPathVertices [llength $ppfPathPrev]
    set ppfCornerPrev [lindex $pppfCorner $uiGeneration]
    set uiCornerVertices [llength $ppfCornerPrev]
    
    set ppfHalf {}
    for { set uiVertex 0 } { $uiVertex < $uiCornerVertices } { incr uiVertex } {
	lappend ppfHalf [MatrixMultMatrix $mHalf [lindex $ppfCornerPrev $uiVertex]]
    }

    for { set uiVertex 0 } { $uiVertex < $uiPathVertices } { incr uiVertex } {
	lappend ppfHalf [lindex $ppfPathPrev $uiVertex]
    }

    ##############################
    # Create the Contour
    set mContour [MatrixRotateZ [expr $SLF_PI]]
    set uiVertices [llength $ppfHalf]
    
    set ppfContour $ppfHalf
    for { set uiVertex 0 } { $uiVertex < $uiVertices } { incr uiVertex } {
 	lappend ppfContour [MatrixMultMatrix $mContour [lindex $ppfHalf $uiVertex]]
    }

    return $ppfContour
}

proc GosperIsland { uiGeneration } {

    global SLF_PI
    set fEdgeScale [expr 1.0/sqrt(7.0)]
    set fAlpha [expr atan2(sqrt(3.0), 5.0)]
    set fBeta [expr $SLF_PI/3.0]

    set pppfPath { { { 0.0 0.0 0.0 1.0 } { 1.0 0.0 0.0 1.0 } } }

    set pmPath {}

    set mScale [MatrixScaleAll $fEdgeScale]
    set mRotate [MatrixRotateZ [expr -$fAlpha]]
    set mTranslate [MatrixTranslate [list 0.0 0.0 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate $mTranslate]]

    set mScale [MatrixScaleAll $fEdgeScale]
    set mRotate [MatrixRotateZ [expr -$fAlpha]]
    set mTranslate [MatrixTranslate [list 1.0 0.0 0.0 0.0]]
    set mRot [MatrixRotateZ [expr $fBeta]]

    lappend pmPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate [MatrixMultMatrix $mTranslate $mRot]]]

    set mScale [MatrixScaleAll [expr 1.0/sqrt(7.0)]]
    set mRotate [MatrixRotateZ [expr $SLF_PI - $fAlpha]]
    set mTranslate [MatrixTranslate [list 1.0 0.0 0.0 0.0]]

    lappend pmPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mRotate $mScale]]

    for { set uiGen 1 } { $uiGen <= $uiGeneration } { incr uiGen } {
	set uiGenPrev [expr $uiGen - 1]

	set pathPrev [lindex $pppfPath $uiGenPrev]
	set uiVertices [llength $pathPrev]
	set pathCurr { { 0.0 0.0 0.0 1.0 } }
	for { set uiSegment 0 } { $uiSegment < 2 } { incr uiSegment } {
	    for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
		lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev $uiVertex]]
	    }
	}

	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix [lindex $pmPath $uiSegment] [lindex $pathPrev [expr $uiVertices - 1 - $uiVertex]]]
	}

	lappend pppfPath $pathCurr
    }

    ##################################################
    # Instance the edge six times to form a contour

    set ppfContour {}
    set ppfPath [lindex $pppfPath $uiGeneration]
    set uiVertices [llength $ppfPath]

    set fTheta 0.0
    set fThetaDelta [expr $SLF_PI/3.0]
    set fPhi [expr 2.0*$SLF_PI/3.0]

    for { set uiSegment 0 } { $uiSegment < 6 } { incr uiSegment } {
	set mRotateA [MatrixRotateZ $fPhi]
	set mTranslate [MatrixTranslate [list 1.0 0.0 0.0 0.0]]
	set mRotateB [MatrixRotateZ $fTheta]

	set mContour [MatrixMultMatrix $mRotateB [MatrixMultMatrix $mTranslate $mRotateA]]

	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    #puts "Vertex %uiVertex: [lindex $ppfPath $uiVertex]]"
	    lappend ppfContour [MatrixMultMatrix $mContour [lindex $ppfPath $uiVertex]]
	}

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return $ppfContour
}

proc Hilbert { uiGeneration } {

    global SLF_PI
    set fScaleCurr 1.0

    set pppfPath { { { 0.0 0.0 0.0 1.0 } { 0.0 1.0 0.0 1.0 } { -1.0 1.0 0.0 1.0 } { -1.0 0.0 0.0 1.0 } } }

    for { set uiGen 1 } { $uiGen <= $uiGeneration } { incr uiGen } {
	set uiGenPrev [expr $uiGen - 1]
	set fScaleNext [expr 2.0*$fScaleCurr + 1.0]

	set fScale [expr $fScaleCurr/$fScaleNext]

	set pathPrev [lindex $pppfPath $uiGenPrev]
	set uiVertices [llength $pathPrev]

	set pathCurr { { 0.0 0.0 0.0 1.0 } }

	set mScale [MatrixScaleAll $fScale]
	set mRotate [MatrixRotateZ [expr $SLF_PI/2.0]]
	set mTranslate [MatrixTranslate [list 1.0 0.0 0.0 0.0]]
	
	set mPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mRotate $mTranslate]]

	for { set uiVertex 1 } { $uiVertex < [expr $uiVertices-1] } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev [expr $uiVertices - 1 - $uiVertex]]]
	}

	if { ( $uiGen & 1 ) } {
	    lappend pathCurr [list [expr 0.0] [expr 0.0 + $fScale] 0.0 1.0]
	} else {
	    lappend pathCurr [list [expr 0.0] [expr 1.0 - $fScale] 0.0 1.0]
	}

	set mTranslateA [MatrixTranslate [list 0.0 -1.0 0.0 0.0]]
	set mScale [MatrixScaleAll $fScale]
	set mTranslateB [MatrixTranslate [list 0.0 1.0 0.0 0.0]]
	
	set mPath [MatrixMultMatrix $mTranslateB [MatrixMultMatrix $mScale $mTranslateA]]

	for { set uiVertex 1 } { $uiVertex < [expr $uiVertices-1] } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	if { ( $uiGen & 1 ) } {
	    lappend pathCurr [list [expr 0.0 - $fScale] [expr 1.0 - $fScale] 0.0 1.0]
	    lappend pathCurr [list [expr -1.0 + $fScale] [expr 1.0 - $fScale] 0.0 1.0]
	}

	set mTranslateA [MatrixTranslate [list 1.0 -1.0 0.0 0.0]]
	set mScale [MatrixScaleAll $fScale]
	set mTranslateB [MatrixTranslate [list -1.0 1.0 0.0 0.0]]
	
	set mPath [MatrixMultMatrix $mTranslateB [MatrixMultMatrix $mScale $mTranslateA]]

	for { set uiVertex 1 } { $uiVertex < [expr $uiVertices-1] } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	if { ( $uiGen & 1 ) } {
	    lappend pathCurr [list [expr -1.0] [expr 0.0 + $fScale] 0.0 1.0]
	} else {
	    lappend pathCurr [list [expr -1.0] [expr 1.0 - $fScale] 0.0 1.0]
	}

	set mScale [MatrixScaleAll $fScale]
	set mRotate [MatrixRotateZ [expr -$SLF_PI/2.0]]
	set mTranslate [MatrixTranslate [list -1.0 0.0 0.0 0.0]]
	
	set mPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mScale $mRotate]]

	for { set uiVertex 1 } { $uiVertex < [expr $uiVertices-1] } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev [expr $uiVertices - 1 - $uiVertex]]]
	}

	lappend pathCurr { -1.0 0.0 0.0 1.0 }

	lappend pppfPath $pathCurr
	set fScaleCurr $fScaleNext
    }

    ##################################################
    # Instance the edge four times to form a contour

    set ppfContour {}
    set ppfPath [lindex $pppfPath $uiGeneration]
    set uiVertices [llength $ppfPath]

    set fTheta [expr $SLF_PI/4.0]
    set fThetaDelta [expr $SLF_PI/2.0]
    set fPhi [expr -$SLF_PI/4.0]

    for { set uiSegment 0 } { $uiSegment < 4 } { incr uiSegment } {
	set mRotateA [MatrixRotateZ $fPhi]
	set mTranslate [MatrixTranslate [list [expr 1.0/sqrt(2.0)] 0.0 0.0 0.0]]
	set mRotateB [MatrixRotateZ $fTheta]

	set mContour [MatrixMultMatrix $mRotateB [MatrixMultMatrix $mTranslate $mRotateA]]

	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    #puts "Vertex $uiVertex: [lindex $ppfPath $uiVertex]]"
	    lappend ppfContour [MatrixMultMatrix $mContour [lindex $ppfPath $uiVertex]]
	}

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return $ppfContour
}

proc Hilbert2 { uiGeneration } {

    global SLF_PI
    set fScaleCurr 2.0

    set pppfPath { { { 0.0 0.0 0.0 1.0 } { 1.0 0.0 0.0 1.0 } { 1.0 0.5 0.0 1.0 } { 0.0 0.5 0.0 1.0 } { 0.0 1.0 0.0 1.0 } { 1.0 1.0 0.0 1.0 } } }

    for { set uiGen 1 } { $uiGen <= $uiGeneration } { incr uiGen } {
	set uiGenPrev [expr $uiGen - 1]
	set fScaleNext [expr 3.0*$fScaleCurr + 2.0]

	set fScale [expr $fScaleCurr/$fScaleNext]

	set pathPrev [lindex $pppfPath $uiGenPrev]
	set uiVertices [llength $pathPrev]

	set pathCurr { { 0.0 0.0 0.0 1.0 } }

	set mScale [MatrixScaleAll $fScale]
	
	set mPath $mScale

	for { set uiVertex 1 } { $uiVertex < [expr $uiVertices-1] } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	set mTranslateA [MatrixTranslate [list 0.0 -1.0 0.0 0.0]]
	set mScale [MatrixScaleAll $fScale]
	set mMirror [MatrixScale {1.0 -1.0 1.0}]
	set mTranslateB [MatrixTranslate [list [expr ($fScaleCurr + 1.0)/$fScaleNext] 0.0 0.0 0.0]]
	
	set mPath [MatrixMultMatrix $mTranslateB [MatrixMultMatrix $mMirror [MatrixMultMatrix $mScale $mTranslateA]]]

	for { set uiVertex 1 } { $uiVertex < [expr $uiVertices-1] } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	set mTranslateA [MatrixTranslate [list -1.0 0.0 0.0 0.0]]
	set mScale [MatrixScaleAll $fScale]
	set mTranslateB [MatrixTranslate [list 1.0 0.0 0.0 0.0]]
	
	set mPath [MatrixMultMatrix $mTranslateB [MatrixMultMatrix $mScale $mTranslateA]]

	for { set uiVertex 1 } { $uiVertex < [expr $uiVertices-1] } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	lappend pathCurr [list 1.0 [expr $fScaleCurr/$fScaleNext] 0.0 1.0]
	lappend pathCurr [list 1.0 [expr ($fScaleCurr + 1.0)/$fScaleNext] 0.0 1.0]

	set mScale [MatrixScaleAll $fScale]
	set mMirror [MatrixScale {-1.0 1.0 1.0}]
	set mTranslate [MatrixTranslate [list 1.0 [expr ($fScaleCurr + 1.0)/$fScaleNext] 0.0 0.0]]
	
	set mPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mMirror $mScale]]

	for { set uiVertex 1 } { $uiVertex < [expr $uiVertices-1] } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	set mScale [MatrixScaleAll $fScale]
	set mRotate [MatrixRotateZ $SLF_PI]
	set mTranslate [MatrixTranslate [list [expr (2.0*$fScaleCurr + 1.0)/$fScaleNext] [expr (2.0*$fScaleCurr + 1.0)/$fScaleNext] 0.0 0.0]]
	
	set mPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mRotate $mScale]]

	for { set uiVertex 1 } { $uiVertex < [expr $uiVertices-1] } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	set mTranslateA [MatrixTranslate [list -1.0 -1.0 0.0 0.0]]
	set mScale [MatrixScaleAll $fScale]
	set mMirror [MatrixScale {-1.0 1.0 1.0}]
	set mTranslateB [MatrixTranslate [list 0.0 [expr (2.0*$fScaleCurr + 1.0)/$fScaleNext] 0.0 0.0]]
	
	set mPath [MatrixMultMatrix $mTranslateB [MatrixMultMatrix $mMirror [MatrixMultMatrix $mScale $mTranslateA]]]

	for { set uiVertex 1 } { $uiVertex < [expr $uiVertices-1] } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	lappend pathCurr [list 0.0 [expr (2.0*$fScaleCurr + 1.0)/$fScaleNext] 0.0 1.0]
	lappend pathCurr [list 0.0 [expr 2.0*($fScaleCurr + 1.0)/$fScaleNext] 0.0 1.0]

	set mTranslateA [MatrixTranslate [list 0.0 -1.0 0.0 0.0]]
	set mScale [MatrixScaleAll $fScale]
	set mTranslateB [MatrixTranslate [list 0.0 1.0 0.0 0.0]]
	
	set mPath [MatrixMultMatrix $mTranslateB [MatrixMultMatrix $mScale $mTranslateA]]

	for { set uiVertex 1 } { $uiVertex < [expr $uiVertices-1] } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	set mTranslateA [MatrixTranslate [list 0.0 -1.0 0.0 0.0]]
	set mScale [MatrixScaleAll $fScale]
	set mMirror [MatrixScale {1.0 -1.0 1.0}]
	set mTranslateB [MatrixTranslate [list [expr ($fScaleCurr + 1.0)/$fScaleNext] [expr 2.0*($fScaleCurr + 1.0)/$fScaleNext] 0.0 0.0]]
	
	set mPath [MatrixMultMatrix $mTranslateB [MatrixMultMatrix $mMirror [MatrixMultMatrix $mScale $mTranslateA]]]

	for { set uiVertex 1 } { $uiVertex < [expr $uiVertices-1] } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	set mTranslateA [MatrixTranslate [list -1.0 -1.0 0.0 0.0]]
	set mScale [MatrixScaleAll $fScale]
	set mTranslateB [MatrixTranslate [list 1.0 1.0 0.0 0.0]]
	
	set mPath [MatrixMultMatrix $mTranslateB [MatrixMultMatrix $mScale $mTranslateA]]

	for { set uiVertex 1 } { $uiVertex < [expr $uiVertices-1] } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	lappend pathCurr [list 1.0 1.0 0.0 1.0]

	lappend pppfPath $pathCurr
	set fScaleCurr $fScaleNext
    }

    ##################################################
    # Instance the edge four times to form a contour

    set ppfContour {}
    set ppfPath [lindex $pppfPath $uiGeneration]
    set uiVertices [llength $ppfPath]

    #set fTheta [expr $SLF_PI/4.0]
    set fTheta 0.0
    set fThetaDelta [expr $SLF_PI/2.0]

    for { set uiSegment 0 } { $uiSegment < 4 } { incr uiSegment } {
	set mRotateA [MatrixRotateZ [expr -$SLF_PI/4.0]]
	set mTranslateA [MatrixTranslate [list [expr 1.0/($fScaleCurr*sqrt(2.0))] 0.0 0.0 0.0]]
	set mScale [MatrixScaleAll [expr 1.0/((1.0+(1.0/$fScaleCurr))*sqrt(2.0))]]
	set mRotateB [MatrixRotateZ [expr 3.0*$SLF_PI/4.0]]
	set mTranslateB [MatrixTranslate [list [expr 1.0/sqrt(2.0)] 0.0 0.0 0.0]]
	set mRotateC [MatrixRotateZ $fTheta]

	set mContour [MatrixMultMatrix $mRotateC [MatrixMultMatrix $mTranslateB [MatrixMultMatrix $mRotateB [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslateA $mRotateA]]]]]

	for { set uiVertex 1 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    #puts "Vertex $uiVertex: [lindex $ppfPath $uiVertex]]"
	    lappend ppfContour [MatrixMultMatrix $mContour [lindex $ppfPath $uiVertex]]
	}

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return $ppfContour
}

proc Sierpinski { uiGeneration } {

    global SLF_PI

    set pppfPath { { { 0.0 0.0 0.0 1.0 } } }
    lappend pppfPath [list [list [expr 1.0/6.0] 0.0 0.0 1.0] [list [expr 1.0/3.0] [expr 1.0/6.0] 0.0 1.0] [list [expr 1.0/2.0] 0.0 0.0 1.0] [list [expr 2.0/3.0] [expr 1.0/6.0] 0.0 1.0] [list [expr 5.0/6.0] 0.0 0.0 1.0] ]

    set fScalePrev 1.0
    set fScaleCurr 3.0
    for { set uiGen 2 } { $uiGen <= $uiGeneration } { incr uiGen } {
	set uiGenPrev [expr $uiGen - 1]
	set fScaleNext [expr 2.0*$fScaleCurr + 1.0]

	set fScale [expr $fScaleCurr/$fScaleNext]

	set pathPrev [lindex $pppfPath $uiGenPrev]
	set uiVertices [llength $pathPrev]

	set pathCurr {}

	set mPath [MatrixScaleAll $fScale]

	for { set uiVertex 0 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	set mScale [MatrixScaleAll $fScale]
	set mTranslate [MatrixTranslate [list 1.0 0.0 0.0 0.0]]
	set mRotate [MatrixRotateZ [expr $SLF_PI/2.0]]
	
	set mPath [MatrixMultMatrix $mScale [MatrixMultMatrix $mTranslate $mRotate]]

	for { set uiVertex 0 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	lappend pathCurr [list 0.5 [expr 2.0*$fScalePrev/$fScaleNext] 0.0 1.0]

	set mTranslate [MatrixTranslate [list [expr 1.0 - $fScale] $fScale 0.0 0.0]]
	set mScale [MatrixScaleAll $fScale]
	set mRotate [MatrixRotateZ [expr -$SLF_PI/2.0]]
	
	set mPath [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mScale $mRotate]]

	for { set uiVertex 0 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	set mScale [MatrixScaleAll $fScale]
	set mTranslate [MatrixTranslate [list [expr 1.0 - $fScale] 0.0 0.0 0.0]]
	
	set mPath [MatrixMultMatrix $mTranslate $mScale]

	for { set uiVertex 0 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    lappend pathCurr [MatrixMultMatrix $mPath [lindex $pathPrev $uiVertex]]
	}

	lappend pppfPath $pathCurr
	set fScalePrev $fScaleCurr
	set fScaleCurr $fScaleNext
    }

    ##################################################
    # Instance the edge four times to form a contour

    set ppfContour {}
    set ppfPath [lindex $pppfPath $uiGeneration]
    set uiVertices [llength $ppfPath]

    set fTheta [expr 0.0]
    set fThetaDelta [expr $SLF_PI/2.0]
    set fPhi [expr 3.0*$SLF_PI/4.0]

    for { set uiSegment 0 } { $uiSegment < 4 } { incr uiSegment } {
	set mRotateA [MatrixRotateZ $fPhi]
	set mTranslate [MatrixTranslate [list [expr 1.0/sqrt(2.0)] 0.0 0.0 0.0]]
	set mRotateB [MatrixRotateZ $fTheta]

	set mContour [MatrixMultMatrix $mRotateB [MatrixMultMatrix $mTranslate $mRotateA]]

	for { set uiVertex 0 } { $uiVertex < $uiVertices } { incr uiVertex } {
	    #puts "Vertex $uiVertex: [lindex $ppfPath $uiVertex]]"
	    lappend ppfContour [MatrixMultMatrix $mContour [lindex $ppfPath $uiVertex]]
	}

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return $ppfContour
}



